/*
 *  linux/kernel/ve.c
 *
 *  Copyright (C) 2000-2005  SWsoft
 *  All rights reserved.
 *
 *  Licensing governed by "linux/COPYING.SWsoft" file.
 *
 */

/*
 * 've.c' helper file performing VE sub-system initialization
 */

#include <linux/sched.h>
#include <linux/delay.h>
#include <linux/capability.h>
#include <linux/ve.h>
#include <linux/smp_lock.h>
#include <linux/init.h>

#include <linux/errno.h>
#include <linux/unistd.h>
#include <linux/slab.h>
#include <linux/sys.h>
#include <linux/kdev_t.h>
#include <linux/termios.h>
#include <linux/tty_driver.h>
#include <linux/netdevice.h>
#include <linux/utsname.h>
#include <linux/proc_fs.h>
#include <linux/kernel_stat.h>
#include <linux/module.h>
#include <linux/rcupdate.h>
#include <linux/ve_proto.h>
#include <linux/ve_owner.h>
#include <linux/devpts_fs.h>

#include <linux/nfcalls.h>

unsigned long vz_rstamp = 0x37e0f59d;

struct module no_module = { .state = MODULE_STATE_GOING };
EXPORT_SYMBOL(no_module);

#ifdef CONFIG_VE

DCL_VE_OWNER(SKB, struct sk_buff, owner_env)
DCL_VE_OWNER(SK, struct sock, sk_owner_env)
DCL_VE_OWNER(TW, struct tcp_tw_bucket, tw_owner_env)
DCL_VE_OWNER(FILP, struct file, owner_env)
DCL_VE_OWNER(FSTYPE, struct file_system_type, owner_env)

#if defined(CONFIG_VE_IPTABLES)
INIT_KSYM_MODULE(x_tables);
INIT_KSYM_MODULE(ip_tables);
INIT_KSYM_MODULE(iptable_filter);
INIT_KSYM_MODULE(iptable_mangle);
INIT_KSYM_MODULE(xt_limit);
INIT_KSYM_MODULE(ipt_multiport);
INIT_KSYM_MODULE(ipt_tos);
INIT_KSYM_MODULE(ipt_TOS);
INIT_KSYM_MODULE(ipt_REJECT);
INIT_KSYM_MODULE(ipt_TCPMSS);
INIT_KSYM_MODULE(xt_tcpmss);
INIT_KSYM_MODULE(ipt_ttl);
INIT_KSYM_MODULE(ipt_LOG);
INIT_KSYM_MODULE(xt_length);
INIT_KSYM_MODULE(ip_conntrack);
INIT_KSYM_MODULE(ip_conntrack_ftp);
INIT_KSYM_MODULE(ip_conntrack_irc);
INIT_KSYM_MODULE(xt_conntrack);
INIT_KSYM_MODULE(xt_state);
INIT_KSYM_MODULE(xt_helper);
INIT_KSYM_MODULE(ip_nat);
INIT_KSYM_MODULE(iptable_nat);
INIT_KSYM_MODULE(ip_nat_ftp);
INIT_KSYM_MODULE(ip_nat_irc);
INIT_KSYM_MODULE(ipt_REDIRECT);

INIT_KSYM_CALL(int, init_netfilter, (void));
INIT_KSYM_CALL(int, init_xtables, (void));
INIT_KSYM_CALL(int, init_iptables, (void));
INIT_KSYM_CALL(int, init_iptable_filter, (void));
INIT_KSYM_CALL(int, init_iptable_mangle, (void));
INIT_KSYM_CALL(int, init_xt_limit, (void));
INIT_KSYM_CALL(int, init_iptable_multiport, (void));
INIT_KSYM_CALL(int, init_iptable_tos, (void));
INIT_KSYM_CALL(int, init_iptable_TOS, (void));
INIT_KSYM_CALL(int, init_iptable_REJECT, (void));
INIT_KSYM_CALL(int, init_iptable_TCPMSS, (void));
INIT_KSYM_CALL(int, init_xt_tcpmss, (void));
INIT_KSYM_CALL(int, init_iptable_ttl, (void));
INIT_KSYM_CALL(int, init_iptable_LOG, (void));
INIT_KSYM_CALL(int, init_xt_length, (void));
INIT_KSYM_CALL(int, init_iptable_conntrack, (void));
INIT_KSYM_CALL(int, init_iptable_ftp, (void));
INIT_KSYM_CALL(int, init_iptable_irc, (void));
INIT_KSYM_CALL(int, init_xt_conntrack_match, (void));
INIT_KSYM_CALL(int, init_xt_state, (void));
INIT_KSYM_CALL(int, init_xt_helper, (void));
INIT_KSYM_CALL(int, ip_nat_init, (void));
INIT_KSYM_CALL(int, init_iptable_nat, (void));
INIT_KSYM_CALL(int, init_iptable_nat_ftp, (void));
INIT_KSYM_CALL(int, init_iptable_nat_irc, (void));
INIT_KSYM_CALL(int, init_iptable_REDIRECT, (void));
INIT_KSYM_CALL(void, fini_iptable_nat_irc, (void));
INIT_KSYM_CALL(void, fini_iptable_nat_ftp, (void));
INIT_KSYM_CALL(void, fini_iptable_nat, (void));
INIT_KSYM_CALL(void, ip_nat_cleanup, (void));
INIT_KSYM_CALL(void, fini_xt_helper, (void));
INIT_KSYM_CALL(void, fini_xt_state, (void));
INIT_KSYM_CALL(void, fini_xt_conntrack_match, (void));
INIT_KSYM_CALL(void, fini_iptable_irc, (void));
INIT_KSYM_CALL(void, fini_iptable_ftp, (void));
INIT_KSYM_CALL(void, fini_iptable_conntrack, (void));
INIT_KSYM_CALL(void, fini_xt_length, (void));
INIT_KSYM_CALL(void, fini_iptable_LOG, (void));
INIT_KSYM_CALL(void, fini_iptable_ttl, (void));
INIT_KSYM_CALL(void, fini_xt_tcpmss, (void));
INIT_KSYM_CALL(void, fini_iptable_TCPMSS, (void));
INIT_KSYM_CALL(void, fini_iptable_REJECT, (void));
INIT_KSYM_CALL(void, fini_iptable_TOS, (void));
INIT_KSYM_CALL(void, fini_iptable_tos, (void));
INIT_KSYM_CALL(void, fini_iptable_multiport, (void));
INIT_KSYM_CALL(void, fini_xt_limit, (void));
INIT_KSYM_CALL(void, fini_iptable_filter, (void));
INIT_KSYM_CALL(void, fini_iptable_mangle, (void));
INIT_KSYM_CALL(void, fini_iptables, (void));
INIT_KSYM_CALL(void, fini_xtables, (void));
INIT_KSYM_CALL(void, fini_netfilter, (void));
INIT_KSYM_CALL(void, fini_iptable_REDIRECT, (void));

INIT_KSYM_CALL(void, ipt_flush_table, (struct xt_table *table));
#endif

#ifdef CONFIG_VE_CALLS_MODULE
INIT_KSYM_MODULE(vzmon);
INIT_KSYM_CALL(int, real_get_device_perms_ve,
		(int dev_type, dev_t dev, int access_mode));
INIT_KSYM_CALL(void, real_do_env_cleanup, (struct ve_struct *env));
INIT_KSYM_CALL(void, real_do_env_free, (struct ve_struct *env));
INIT_KSYM_CALL(void, real_update_load_avg_ve, (void));

int get_device_perms_ve(int dev_type, dev_t dev, int access_mode)
{
	return KSYMSAFECALL(int, vzmon, real_get_device_perms_ve,
					(dev_type, dev, access_mode));
}
EXPORT_SYMBOL(get_device_perms_ve);

void do_env_cleanup(struct ve_struct *env)
{
	KSYMSAFECALL_VOID(vzmon, real_do_env_cleanup, (env));
}

void do_env_free(struct ve_struct *env)
{
	KSYMSAFECALL_VOID(vzmon, real_do_env_free, (env));
}
EXPORT_SYMBOL(do_env_free);

void do_update_load_avg_ve(void)
{
	KSYMSAFECALL_VOID(vzmon, real_update_load_avg_ve, ());
}
#endif

struct ve_struct ve0 = {
	.utsname		= &system_utsname,
	.vetask_lh		= LIST_HEAD_INIT(ve0.vetask_lh),
#if defined(CONFIG_VE_NETDEV) || defined(CONFIG_VE_NETDEV_MODULE)
	._net_dev_tail		= &ve0._net_dev_base,
	.ifindex		= -1,
#endif
#ifdef CONFIG_UNIX98_PTYS
	.devpts_config		= &devpts_config,
#endif
};

EXPORT_SYMBOL(ve0);

#endif /* CONFIG_VE */
