/* Kernel module to match TOS values. */

/* (C) 1999-2001 Paul `Rusty' Russell
 * (C) 2002-2004 Netfilter Core Team <coreteam@netfilter.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/nfcalls.h>

#include <linux/netfilter_ipv4/ipt_tos.h>
#include <linux/netfilter_ipv4/ip_tables.h>

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("iptables TOS match module");

#ifdef CONFIG_VE_IPTABLES
#include <linux/sched.h>
#define ve_tos_match		(*(get_exec_env()->_tos_match))
#else
#define ve_tos_match		tos_match
#endif

static int
match(const struct sk_buff *skb,
      const struct net_device *in,
      const struct net_device *out,
      const void *matchinfo,
      int offset,
      unsigned int protoff,
      int *hotdrop)
{
	const struct ipt_tos_info *info = matchinfo;

	return (skb->nh.iph->tos == info->tos) ^ info->invert;
}

static int
checkentry(const char *tablename,
           const void *ip,
           void *matchinfo,
           unsigned int matchsize,
           unsigned int hook_mask)
{
	if (matchsize != IPT_ALIGN(sizeof(struct ipt_tos_info)))
		return 0;

	return 1;
}

#ifdef CONFIG_COMPAT
static int compat(void *match,
		void **dstptr, int *size, int convert)
{
	int off;

	off = IPT_ALIGN(sizeof(struct ipt_tos_info)) -
		COMPAT_IPT_ALIGN(sizeof(struct ipt_tos_info));
	return ipt_match_align_compat(match, dstptr, size, off, convert);
}
#endif

static struct ipt_match tos_match = {
	.name		= "tos",
	.match		= &match,
	.checkentry	= &checkentry,
#ifdef CONFIG_COMPAT
	.compat		= &compat,
#endif
	.me		= THIS_MODULE,
};

int init_iptable_tos(void)
{
	return ipt_register_match(&tos_match);
}

void fini_iptable_tos(void)
{
	ipt_unregister_match(&tos_match);
}

static int __init init(void)
{
	int err;

	err = init_iptable_tos();
	if (err < 0)
		return err;

	KSYMRESOLVE(init_iptable_tos);
	KSYMRESOLVE(fini_iptable_tos);
	KSYMMODRESOLVE(ipt_tos);
	return 0;
}

static void __exit fini(void)
{
	KSYMMODUNRESOLVE(ipt_tos);
	KSYMUNRESOLVE(init_iptable_tos);
	KSYMUNRESOLVE(fini_iptable_tos);
	fini_iptable_tos();
}

module_init(init);
module_exit(fini);
